@@ -25,13 +25,24 @@
 class QColor;
 class KConfig;
 class BosonConfig;
+class BoVector3;
+class BoVector4;
 template<class T> class QValueList;
 
 #define boConfig BosonConfig::bosonConfig()
 
-
 class BoConfigEntry
 {
+public:
+	enum Type {
+		Bool = 0,
+		Int = 1,
+		UInt = 2,
+		Double = 3,
+		String = 4,
+		IntList = 5,
+		Color = 6
+	};
 public:
 	BoConfigEntry(BosonConfig* parent, const QString& key);
 	virtual ~BoConfigEntry();
@@ -48,6 +59,8 @@ public:
 	virtual void save(KConfig* conf) = 0;
 	virtual void load(KConfig* conf) = 0;
 
+	virtual int type() const = 0;
+
 	const QString& key() const { return mKey; }
 
 private:
@@ -67,6 +80,8 @@ public:
 	virtual void save(KConfig* conf);
 	virtual void load(KConfig* conf);
 
+	virtual int type() const { return Bool; }
+
 private:
 	bool mValue;
 };
@@ -83,6 +98,8 @@ public:
 	virtual void save(KConfig* conf);
 	virtual void load(KConfig* conf);
 
+	virtual int type() const { return Int; }
+
 private:
 	int mValue;
 };
@@ -100,6 +117,8 @@ public:
 	virtual void save(KConfig* conf);
 	virtual void load(KConfig* conf);
 
+	virtual int type() const { return UInt; }
+
 private:
 	unsigned int mValue;
 };
@@ -116,6 +135,8 @@ public:
 	virtual void save(KConfig* conf);
 	virtual void load(KConfig* conf);
 
+	virtual int type() const { return Double; }
+
 private:
 	double mValue;
 };
@@ -132,10 +153,31 @@ public:
 	virtual void save(KConfig* conf);
 	virtual void load(KConfig* conf);
 
+	virtual int type() const { return String; }
+
 private:
 	QString mValue;
 };
 
+class BoConfigColorEntry : public BoConfigEntry
+{
+public:
+	BoConfigColorEntry(BosonConfig* parent, const QString& key, const QColor& defaultValue);
+	virtual ~BoConfigColorEntry() {}
+
+	QColor value() const;
+	void setValue(unsigned int rgb ) { mRGBValue = rgb; }
+	void setValue(const QColor& v);
+
+	virtual void save(KConfig* conf);
+	virtual void load(KConfig* conf);
+
+	virtual int type() const { return Color; }
+
+private:
+	unsigned int mRGBValue;
+};
+
 class BoConfigIntListEntry; // forwarding, since i dont want to #include <qvaluelist.h>
 
 /**
@@ -173,9 +215,40 @@ public:
 		DebugSelection = 1
 	};
 
-	static BosonConfig* bosonConfig() { return mBosonConfig; }
-	
-	static void initBosonConfig();
+	static BosonConfig* bosonConfig();
+
+	/**
+	 * Set a function that will be called immediately after @ref
+	 * BoGlobalObjectBase::loadObject() has returned successfully.
+	 *
+	 * You can use this to do some initialization for other global objects,
+	 * that will get constructed after the BosonConfig object, such as
+	 * applying the --nosound parameter (will get used by @ref BosonMusic
+	 * constructor)
+	 **/
+	static void setPostInitFunction(void (*func)());
+
+	/**
+	 * Read and return a @ref BoVector3. Will return @p aDefault if @p key
+	 * is not found.
+	 **/
+	static BoVector3 readBoVector3Entry(const KConfig* cfg, const QString& key, const BoVector3& aDefault);
+
+	/**
+	 * @overload
+	 *
+	 * Just the same as the above one, but it uses BoVector3(0, 0, 0) as
+	 * default. We don't use a default value in the parameter, as we would
+	 * have to include bo3dtools.h for that.
+	 **/
+	static BoVector3 readBoVector3Entry(const KConfig* cfg, const QString& key);
+
+	static BoVector4 readBoVector4Entry(const KConfig* cfg, const QString& key, const BoVector4& aDefault);
+	static BoVector4 readBoVector4Entry(const KConfig* cfg, const QString& key);
+
+	static void writeEntry(KConfig* cfg, const QString& key, const BoVector3& value);
+	static void writeEntry(KConfig* cfg, const QString& key, const BoVector4& value);
+
 
 	static QString readLocalPlayerName(KConfig* conf = 0);
 	static void saveLocalPlayerName(const QString& name, KConfig* conf = 0);
@@ -192,6 +265,9 @@ public:
 	static QString readEditorMap(KConfig* conf = 0);
 	static void saveEditorMap(const QString& id, KConfig* conf = 0);
 
+	static bool readEditorCreateNewMap(KConfig* conf = 0);
+	static void saveEditorCreateNewMap(bool createnew, KConfig* conf = 0);
+
 	static void saveGameSpeed(int speed, KConfig* conf = 0);
 	static int readGameSpeed(KConfig* conf = 0);
 
@@ -218,8 +294,6 @@ public:
 	double miniMapScale() const { return mMiniMapScale->value(); }
 	void setMiniMapZoom(double z) { mMiniMapZoom->setValue(z); }
 	double miniMapZoom() const { return mMiniMapZoom->value(); }
-	void setCommandButtonsPerRow(int b) { mCommandButtonsPerRow->setValue(b); }
-	int commandButtonsPerRow() const { return mCommandButtonsPerRow->value(); }
 	void setChatScreenRemoveTime(unsigned int s) { mChatScreenRemoveTime->setValue(s); }
 	unsigned int chatScreenRemoveTime() const { return mChatScreenRemoveTime->value(); }
 	void setChatScreenMaxItems(int max) { mChatScreenMaxItems->setValue(max); }
@@ -242,6 +316,8 @@ public:
 	CameraAction mouseWheelShiftAction() const { return (CameraAction)(mMouseWheelShiftAction->value()); }
 	void setDeactivateWeaponSounds(bool deactivate) { mDeactivateWeaponSounds->setValue(deactivate); }
 	bool deactivateWeaponSounds() const { return mDeactivateWeaponSounds->value(); }
+	void setUseLOD(bool use) { mUseLOD->setValue(use); }
+	bool useLOD() const { return mUseLOD->value(); }
 
 	void setUnitSoundActivated(UnitSoundEvent e, bool activated);
 	bool unitSoundActivated(UnitSoundEvent e) const;
@@ -256,6 +332,9 @@ public:
 	bool useLight() const { return mUseLight->value(); }
 	void setUseLight(bool l) const { return mUseLight->setValue(l); }
 
+	bool useMaterials() const { return mUseMaterials->value(); }
+	void setUseMaterials(bool l) const { return mUseMaterials->setValue(l); }
+
 	void setCursorMode(int mode) { mCursorMode->setValue(mode); }
 	int cursorMode() const { return mCursorMode->value(); }
 
@@ -268,6 +347,122 @@ public:
 	void setToolTipCreator(int type) { mToolTipCreator->setValue(type); }
 	int toolTipCreator() const { return mToolTipCreator->value(); }
 
+	void setGameLogInterval(int interval) { mGameLogInterval->setValue(interval); }
+	int gameLogInterval() const { return mGameLogInterval->value(); }
+
+
+
+
+
+	/**
+	 * Add a dynamic entry to BosonConfig. A dynamic entry does not have a
+	 * dedicated set/get function, but rather uses @ref setBoolValue,
+	 * @ref value and friends.
+	 *
+	 * You should not use this for extremely time critical config entries
+	 * (time critical means it is used several dozen times per second at
+	 * least), as at least one @ref QDict lookup is involved for nearly
+	 * every operation.
+	 *
+	 * For all non-time critical config entries this can be very handy, as
+	 * you don't need to modify bosonconfig.h (and therefore don't have to
+	 * recompile everything).
+	 * @param conf The @ref KConfig object that is looked into for whether
+	 * the key already has a value.
+	 **/
+	void addDynamicEntry(BoConfigEntry* entry, KConfig* conf = 0);
+
+	/**
+	 * @return TRUE if a dynamic entry with @p key was added with @ref
+	 * addDynamicEntry, otherwise FALSE.
+	 **/
+	bool hasKey(const QString& key) const;
+
+	/**
+	 * Set the value of the config entry @p key (see @ref value) to @p
+	 * value. This operates on @ref BoConfigBoolEntry objects.
+	 *
+	 * Note that an overloaded "setValue()" method for this task is not
+	 * provided, because while programming it is only a minor difference
+	 * between setUIntValue() and setIntValue(), but as they resolve to two
+	 * different classes in BosonConfig it is a major difference here. I
+	 * believe this is not a problem for us.
+	 **/
+	void setBoolValue(const QString& key, bool value);
+
+	/**
+	 * Same as  @ref setBoolValue, but for @ref BoConfigIntEntry objects.
+	 **/
+	void setIntValue(const QString& key, int value);
+
+	/**
+	 * Same as  @ref setBoolValue, but for @ref BoConfigUIntEntry objects.
+	 **/
+	void setUIntValue(const QString& key, unsigned int value);
+
+	/**
+	 * Same as  @ref setBoolValue, but for @ref BoConfigDoubleEntry objects.
+	 **/
+	void setDoubleValue(const QString& key, double value);
+
+	/**
+	 * Same as  @ref setBoolValue, but for @ref BoConfigStringEntry objects.
+	 **/
+	void setStringValue(const QString& key, const QString& value);
+
+	/**
+	 * Same as  @ref setBoolValue, but for @ref BoConfigColorEntry objects.
+	 **/
+	void setColorValue(const QString& key, const QColor& value);
+
+	/**
+	 * @return The (dynamic) entry for @p key, or NULL if no such key was
+	 * ever added using @ref addDynamicEntry.
+	 **/
+	BoConfigEntry* value(const QString& key) const;
+
+	/**
+	 * @return The value of the entry @p key, or @p _default no entry of the
+	 * correct type exists.
+	 **/
+	bool boolValue(const QString& key, bool _default = false) const;
+
+	/**
+	 * @return The value of the entry @p key, or @p _default no entry of the
+	 * correct type exists.
+	 **/
+	int intValue(const QString& key, int _default = 0) const;
+
+	/**
+	 * @return The value of the entry @p key, or @p _default no entry of the
+	 * correct type exists.
+	 **/
+	unsigned int uintValue(const QString& key, unsigned int _default = 0) const;
+
+	/**
+	 * @return The value of the entry @p key, or @p _default no entry of the
+	 * correct type exists.
+	 **/
+	const QString& stringValue(const QString& key, const QString& _default = QString::null) const;
+
+	/**
+	 * @return The value of the entry @p key, or @p _default no entry of the
+	 * correct type exists.
+	 **/
+	double doubleValue(const QString& key, double _default = 0.0) const;
+
+	/**
+	 * @return The value of the entry @p key, or @ref Qt::black if no entry
+	 * of the correct type exists.
+	 **/
+	QColor colorValue(const QString& key) const;
+
+	/**
+	 * @return The value of the entry @p key, or @p _default if no entry of
+	 * the correct type exists.
+	 **/
+	QColor colorValue(const QString& key, const QColor& _default) const;
+
 
 // below we have config values that are *not* stored when quitting boson
 public:
@@ -291,19 +486,41 @@ public:
 	 **/
 	bool disableSound() const { return mDisableSound; }
 
-	void setLoadTiles(bool l) { mLoadTiles = l; }
-
-	/**
-	 * If FALSE (for debugging only) then no tiles are loaded (or dummy
-	 * tiles only). Useful for runtime profiling of advance stuff for
-	 * example, since we don't need to wait for tiles to be loaded.
-	 *
-	 * Of course rendering values are useless then.
-	 **/
-	bool loadTiles() const { return mLoadTiles; }
-
-	float aiDelay() const { return mAIDelay; }
 	void setAiDelay(float delay) { mAIDelay = delay; }
+	float aiDelay() const { return mAIDelay; }
+
+	void setWantDirect(bool w) { mWantDirect = w; }
+	bool wantDirect() const { return mWantDirect; }
+
+	void setWireFrames(bool on) { mWireFrames = on; }
+	bool wireFrames() const { return mWireFrames; }
+
+	void setDebugOpenGLMatrices(bool debug) { mDebugOpenGLMatrices = debug; }
+	bool debugOpenGLMatrices() const { return mDebugOpenGLMatrices; }
+
+	void setDebugMapCoordinates(bool debug) { mDebugMapCoordinates = debug; }
+	bool debugMapCoordinates() const { return mDebugMapCoordinates; }
+
+	void setDebugShowCellGrid(bool debug) { mDebugShowCellGrid = debug; }
+	bool debugShowCellGrid() const { return mDebugShowCellGrid; }
+
+	void setDebugItemWorkStatistics(bool debug) { mDebugItemWorkStatistics = debug; }
+	bool debugItemWorkStatistics() const { return mDebugItemWorkStatistics; }
+
+	void setDebugOpenGLCamera(bool debug) { mDebugOpenGLCamera = debug; }
+	bool debugOpenGLCamera() const { return mDebugOpenGLCamera; }
+
+	void setDebugRenderCounts(bool debug) { mDebugRenderCounts = debug; }
+	bool debugRenderCounts() const { return mDebugRenderCounts; }
+
+	void setDebugBoundingBoxes(bool debug) { mDebugBoundingBoxes = debug; }
+	bool debugBoundingBoxes() const { return mDebugBoundingBoxes; }
+
+	void setDebugFPS(bool debug) { mDebugFPS = debug; }
+	bool debugFPS() const { return mDebugFPS; }
+
+	void setShowResources(bool show) { mShowResources = show; }
+	bool showResources() const { return mShowResources; }
 
 public:
 	void save(bool editor = false, KConfig* conf = 0);
@@ -325,13 +542,7 @@ public:
 	static void writeFloatNumList(QValueList<float> list, KConfig* cfg, const QString key);
 
 
-protected:
-	int readCommandButtonsPerRow(KConfig* conf);
-	void saveCommandButtonsPerRow(KConfig* conf);
-
 private:
-	static BosonConfig* mBosonConfig;
-	
 	class BosonConfigPrivate;
 	BosonConfigPrivate* d;
 
@@ -344,7 +555,6 @@ private:
 	BoConfigBoolEntry* mShowMenubarInGame;
 	BoConfigBoolEntry* mShowMenubarOnStartup;
 	BoConfigBoolEntry* mModelTexturesMipmaps;
-	BoConfigIntEntry* mCommandButtonsPerRow;
 	BoConfigIntEntry* mChatScreenMaxItems;
 	BoConfigUIntEntry* mChatScreenRemoveTime;
 	BoConfigUIntEntry* mArrowKeyStep;
@@ -362,16 +572,29 @@ private:
 	BoConfigIntEntry* mMouseWheelShiftAction;
 	BoConfigBoolEntry* mDeactivateWeaponSounds;
 	BoConfigBoolEntry* mUseLight;
+	BoConfigBoolEntry* mUseMaterials;
 	BoConfigStringEntry* mCursorDir;
 	BoConfigIntEntry* mCursorMode;
 	BoConfigIntEntry* mToolTipUpdatePeriod;
 	BoConfigIntEntry* mToolTipCreator;
+	BoConfigIntEntry* mGameLogInterval;
+	BoConfigBoolEntry* mUseLOD;
 
 	// NOT stored to config file!
 	bool mDisableSound;
 	DebugMode mDebugMode;
-	bool mLoadTiles;
 	float mAIDelay;
+	bool mWantDirect;
+	bool mWireFrames;
+	bool mDebugOpenGLMatrices;
+	bool mDebugMapCoordinates;
+	bool mDebugShowCellGrid;
+	bool mDebugItemWorkStatistics;
+	bool mDebugOpenGLCamera;
+	bool mDebugRenderCounts;
+	bool mDebugBoundingBoxes;
+	bool mDebugFPS;
+	bool mShowResources;
 };
 
 #endif
