@@ -1,6 +1,6 @@
 /*
     This file is part of the Boson game
-    Copyright (C) 2002 The Boson Team (boson-devel@lists.sourceforge.net)
+    Copyright (C) 2002-2003 The Boson Team (boson-devel@lists.sourceforge.net)
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -21,7 +21,6 @@
 
 #include <GL/gl.h>
 
-class BosonModelTextures;
 class KSimpleConfig;
 class QColor;
 class QString;
@@ -29,6 +28,8 @@ class QStringList;
 class BoVector3;
 class BoMatrix;
 class BoMesh;
+class BoMaterial;
+class BosonModelLoaderData;
 template<class T> class QPtrList;
 template<class T, class T2> class QMap;
 template<class T> class QIntDict;
@@ -71,7 +72,15 @@ public:
 	 * This can be used for the construction animations. Remember to set the
 	 * correct display lists!
 	 **/
-	BoFrame(const BoFrame& frame, int firstMesh, int meshCount);
+	BoFrame(const BoFrame& frame, unsigned int firstMesh, unsigned int meshCount);
+
+	/**
+	 * @overload
+	 * Just as above, but this version takes an array of mesh indices from
+	 * @p frame that will get copied into thuis frame.
+	 **/
+	BoFrame(const BoFrame& frame, unsigned int* meshes, unsigned int meshCount);
+
 	~BoFrame();
 
 	/**
@@ -80,7 +89,7 @@ public:
 	 **/
 	void allocMeshes(int meshes);
 
-	void setMesh(int index, BoMesh* mesh);
+	void setMesh(unsigned int index, BoMesh* mesh);
 
 	/**
 	 * To initialize the mesh you should call @ref setMesh before calling
@@ -99,6 +108,22 @@ public:
 	 **/
 	BoMatrix* matrix(int index) const;
 
+	/**
+	 * Mark the mesh at @p indes (see @ref setMesh) as hidden, if @p hidden
+	 * is TRUE. It will not be rendered, unless this is called again with @p
+	 * hidden = FALSE (the default).
+	 **/
+	void setHidden(unsigned int index, bool hidden);
+
+	/**
+	 * @return Whether the mesh is hidden (i.e. should not be rendered) or
+	 * not. By default it is not hidden.
+	 */
+	inline bool hidden(unsigned int index) const
+	{
+		return mHidden[index];
+	}
+
 	void setDisplayList(GLuint l) { mDisplayList = l; }
 	GLuint displayList() const { return mDisplayList; }
 	float depthMultiplier() const { return mDepthMultiplier; }
@@ -109,26 +134,32 @@ public:
 	//*different* meshes! we can have several pointers two the same mesh,
 	//but they will have different matrices. e.g. we can have 4 wheels (-->
 	//the same mesh, but 4 different positions)
-	int meshCount() const { return mMeshCount; }
+	unsigned int meshCount() const { return mMeshCount; }
 
-	void renderFrame(const QColor* teamColor);
+	/**
+	 * @param lod See @ref BoMesh::renderMesh
+	 **/
+	void renderFrame(const QColor* teamColor, unsigned int lod = 0, int mode = GL_RENDER);
 
 	void mergeMeshes();
 	void sortByDepth();
 
 private:
 	void init();
+	void copyMeshes(const BoFrame& frame, unsigned int* meshes, unsigned int count);
 
 private:
 	GLuint mDisplayList;
 	float mDepthMultiplier;
 	float mRadius; // TODO
 
-	int mMeshCount;
+	unsigned int mMeshCount;
 	BoMatrix** mMatrices;
 	BoMesh** mMeshes;
+	bool* mHidden;
 };
 
+class BosonModelPrivate;
 /**
  * @author Andreas Beckermann <b_mann@gmx.de>
  */
@@ -145,26 +176,34 @@ public:
 	~BosonModel();
 
 	/**
-	 * Add a mesh. This class takes ownership of that mesh, i.e. will delete
-	 * it when this class gets deleted.
+	 * @return The width of the model in the OpenGL coordinate system. This
+	 * is the value provided in the constructor - most units will use 1.0 or
+	 * 1.5 or 2.0 probably.
 	 **/
-	void addMesh(BoMesh* mesh);
+	float width() const { return mWidth; }
 
 	/**
-	 * Set the texture for @p mesh. The filename will be changed by @ref
-	 * BosonModel::cleanTextureName before it will get loaded.
+	 * @return The height of the model in the OpenGL coordinate system. This
+	 * is the value provided in the constructor - most units will use 1.0 or
+	 * 1.5 or 2.0 probably.
 	 **/
-	void setTexture(BoMesh* mesh, const QString& texture);
+	float height() const { return mHeight; }
 
 	/**
-	 * Add @ count frames. You need to give them useful data using @ref
-	 * frame
-	 * @return The offset after which the frames where added. E.g. you can
-	 * retrive the first newly added frame using frame(offset + 0)
+	 * Allocate @p count material objects. You must call this before you can
+	 * use @ref setMaterial and @ref material.
+	 *
+	 * The material objects will be initialized as "default" materials.
 	 **/
-	int addFrames(int count);
+	void allocateMaterials(unsigned int count);
 
-	BoMesh* mesh(int index) const;
+	/**
+	 * @return The material object at @p index.
+	 **/
+	BoMaterial* material(unsigned int index) const;
+	unsigned int materialCount() const;
+
+	BoMesh* mesh(unsigned int index) const;
 	unsigned int meshCount() const;
 	QIntDict<BoMesh> allMeshes() const;
 
@@ -219,8 +258,6 @@ public:
 	 **/
 	void setLongNames(QMap<QString, QString> names);
 
-	static void reloadAllTextures();
-
 	/**
 	 * Add an animation mode with the identitfier @p mode, starting at frame
 	 * number @p start with the number of frames @p range. The frame should
@@ -275,11 +312,19 @@ public:
 	 **/
 	void enablePointer();
 
+	/**
+	 * @return How many LOD levels model has by default
+	 */
+	static unsigned int defaultLodCount();
 
 protected:
 	class BoHelper; // for computing width,height,.. of the model. this is a hack!
 
 protected:
+	/**
+	 * Load the data from @p data into this class.
+	 **/
+	bool loadModelData(BosonModelLoaderData* data);
 	void loadTextures(const QStringList& textures);
 
 	/**
@@ -298,6 +343,12 @@ protected:
 
 	void computeBoundings(BoFrame* frame, BoHelper* helper) const;
 
+	/**
+	 * Compute bounding objects for all meshes and all frames. Usually these
+	 * objects will be boxes.
+	 **/
+	void computeBoundingObjects();
+
 	/**
 	 * Convert a 3ds texture name to a clean name. That means call
 	 * QString::lower() on it, currently. It'll also map the "short name"
@@ -312,6 +363,8 @@ protected:
 	 **/
 	const QString& baseDirectory() const;
 
+	void generateLOD();
+
 	/**
 	 * @ref BoMesh allocates its own array(s) for vertices and texture
 	 * coordinates. It is more efficient to maintain a single array that
@@ -337,13 +390,10 @@ private:
 	void init();
 
 private:
-	class Private;
 	friend class KGameModelDebug;
 
 private:
-	static BosonModelTextures* mModelTextures;
-
-	Private* d;
+	BosonModelPrivate* d;
 
 	float mWidth;
 	float mHeight;
