@@ -20,7 +20,6 @@
 #define UNIT_H
 
 #include "unitbase.h"
-#include "items/bosonitem.h"
 #include "global.h"
 
 class Player;
@@ -51,7 +50,7 @@ class KGameUnitDebug;
  * therefore not possible! This is done to save as much memory as possible.
  * @author Thomas Capricelli <capricel@email.enst.fr>, Andreas Beckermann <b_mann@gmx.de>
  **/
-class Unit : public UnitBase, public BosonItem
+class Unit : public UnitBase
 {
 public:
 	enum PropertyIds {
@@ -63,9 +62,9 @@ public:
 		IdWantedRotation = UnitBase::IdLast + 6,
 		IdMoveAttacking = UnitBase::IdLast + 7,
 		IdSearchPath = UnitBase::IdLast + 8,
+		IdSlowDownAtDestination = UnitBase::IdLast + 9,
 
 		// properties in MobileUnit
-		IdSpeed = UnitBase::IdLast + 50,
 		IdMovingFailed = UnitBase::IdLast + 51,
 		IdPathRecalculated = UnitBase::IdLast + 52,
 		IdPathAge = UnitBase::IdLast + 53,
@@ -151,7 +150,6 @@ public:
 	inline virtual void advanceFunction(unsigned int advanceCount)
 	{
 		(this->*mAdvanceFunction)(advanceCount);
-		mAdvanceFunction = mAdvanceFunction2;
 	}
 
 	/**
@@ -166,6 +164,28 @@ public:
 	inline virtual void advanceFunction2(unsigned int advanceCount)
 	{
 		(this->*mAdvanceFunction2)(advanceCount);
+	}
+
+	
+	/**
+	 * Synchronize the advance function variable. This needs to be called
+	 * after @ref advanceFunction has been called for <em>all</em> units.
+	 *
+	 * Use @ref syncAdvanceFunction2 when @ref advanceFunction2 was called
+	 * on all units.
+	 *
+	 * DO NOT CALL THIS unless you REALLY know what youre doing! Call it
+	 * from @ref BosonCanvas::slotAdvance ONLY!
+	 **/
+	inline virtual void syncAdvanceFunction()
+	{
+		mAdvanceFunction = mAdvanceFunction2;
+	}
+	/**
+	 * See @ref syncAdvanceFunction
+	 **/
+	inline virtual void syncAdvanceFunction2()
+	{
 		mAdvanceFunction2 = mAdvanceFunction;
 	}
 
@@ -228,7 +248,7 @@ public:
 	Unit* target() const;
 	virtual void setTarget(Unit* target);
 
-	bool inRange(long unsigned int, Unit* unit) const;
+	bool inRange(unsigned long int, Unit* unit) const;
 
 // waypoint stuff: // also in facility - produced units receive this initial waypoint
 	/**
@@ -286,9 +306,10 @@ public:
 	 * occupied and you should use e.g. @ref weaponRange if the unit should
 	 * attack. -1 keeps the previously set range.
 	 * @param attack Whether to stop and attack any enemy units in range while moving
+	 * @param slowDownAtDetination Whether to slow down before reaching detination point
 	 * @return true if unit can go to destination, false otherwise
 	 **/
-	bool moveTo(float x, float y, int range = 0, bool attack = false);
+	bool moveTo(float x, float y, int range = 0, bool attack = false, bool slowDownAtDetination = true);
 
 	/**
 	 * Turns unit smoothly to given degrees
@@ -304,8 +325,6 @@ public:
 
 	virtual bool saveAsXML(QDomElement& root);
 	virtual bool loadFromXML(const QDomElement& root);
-	virtual bool save(QDataStream& stream);
-	virtual bool load(QDataStream& stream);
 
 	void updateSelectBox();
 
@@ -370,6 +389,11 @@ public:
 	 **/
 	int distance(const BoVector3& pos) const;
 
+	/**
+	 * @return Weapon with given id for this unit. 0 if it doesn't exist.
+	 **/
+	BosonWeapon* weapon(unsigned long int id) const;
+
 
 protected:
 	void shootAt(BosonWeapon* w, Unit* target);
@@ -388,7 +412,7 @@ protected:
 	 * This is in Unit instead of @ref MobileUnit so that we can apply a
 	 * path to newly constructed units of factories.
 	 **/
-	void newPath();
+	virtual void newPath();
 
 	/**
 	 * Move the unit. By default this does nothing. Reimplemented in @ref
@@ -407,6 +431,8 @@ protected:
 	 **/
 	int moveAttacking() const;
 
+	int slowDownAtDestination() const;
+
 	/**
 	 * @return Whether new path should be searched
 	 **/
@@ -464,9 +490,6 @@ public:
 	MobileUnit(const UnitProperties* prop, Player* owner, BosonCanvas* canvas);
 	virtual ~MobileUnit();
 
-	virtual void setSpeed(float s);
-	virtual float speed() const;
-
 	/**
 	 * Turn to direction. This sets a new frame according to the new
 	 * direction.
@@ -492,11 +515,19 @@ public:
 
 	virtual bool saveAsXML(QDomElement& root);
 	virtual bool loadFromXML(const QDomElement& root);
-	virtual bool load(QDataStream& stream);
-	virtual bool save(QDataStream& stream);
 
 	virtual void stopMoving();
 
+	bool attackEnemyUnitsInRangeWhileMoving();
+
+	virtual void newPath();
+
+	/**
+	 * Check if waypoint wp marks end of the path. If yes, then it stops unit,
+	 * turns to random direction and true, otherwise returns false.
+	 **/
+	bool checkWaypoint(const QPoint& wp);
+
 protected:
 	virtual void advanceMoveInternal(unsigned int advanceCount); // move one step futher to path
 
@@ -587,8 +618,6 @@ public:
 
 	virtual bool saveAsXML(QDomElement& root);
 	virtual bool loadFromXML(const QDomElement& root);
-	virtual bool load(QDataStream& stream);
-	virtual bool save(QDataStream& stream);
 
 private:
 	class FacilityPrivate;
