@@ -31,9 +31,13 @@
 #include "cell.h"
 #include "upgradeproperties.h"
 #include "bodebug.h"
+#include "bosonweapon.h"
 
 #include <klocale.h>
 
+#include <qpair.h>
+#include <qpoint.h>
+
 UnitPlugin::UnitPlugin(Unit* unit)
 {
  mUnit = unit;
@@ -277,7 +281,7 @@ void ProductionPlugin::advance(unsigned int)
 				}
 			}
 		}
-		boDebug() << "Cannot find free cell around facility :-(" << endl;
+		boDebug() << k_funcinfo << "Cannot find free cell around facility :-(" << endl;
 		if (player() == boGame->localPlayer()) {
 			game()->slotAddChatSystemMessage(i18n("%1 could not be placed on the map - no free cell found. Place it manuall!").arg(prop->name()));
 		}
@@ -308,14 +312,14 @@ void RepairPlugin::repair(Unit* u)
  boDebug() << k_funcinfo << endl;
  if (unit()->isFacility()) {
 	if (!u->moveTo(unit()->x(), unit()->y(), 1)) {
-		boDebug() << u->id() << " cannot find a way to repairyard" << endl;
+		boDebug() << k_funcinfo << u->id() << " cannot find a way to repairyard" << endl;
 		u->setWork(Unit::WorkNone);
 	} else {
 		u->setWork(Unit::WorkMove);
 	}
  } else {
 	if (!unit()->moveTo(u->x(), u->y(), 1)) {
-		boDebug() << "Cannot find way to " << u->id() << endl;
+		boDebug() << k_funcinfo << "Cannot find way to " << u->id() << endl;
 		unit()->setWork(Unit::WorkNone);
 	} else {
 		unit()->setAdvanceWork(Unit::WorkMove);
@@ -337,7 +341,7 @@ void RepairPlugin::repairInRange()
 		continue;
 	}
 	if (!player()->isEnemy(u->owner())) {
-		boDebug() << "repair " << u->id() << endl;
+		boDebug() << k_funcinfo << "repair " << u->id() << endl;
 		int diff = u->health() - u->unitProperties()->health();
 		if (diff > 0) {
 			boError() << k_funcinfo << "health > maxhealth" << endl;
@@ -412,7 +416,7 @@ void HarvesterPlugin::advanceMine()
 		} else if (prop->canMineOil()) {
 			player()->statistics()->increaseMinedOil(step);
 		}
-		boDebug() << "resources mined: " << resourcesMined() << endl;
+		boDebug() << k_funcinfo << "resources mined: " << resourcesMined() << endl;
 	} else {
 		// This unit cannot mine at this cell
 		boDebug() << k_funcinfo << "cannot mine here" << endl;
@@ -567,11 +571,11 @@ void HarvesterPlugin::setRefinery(Unit* refinery)
 
 bool HarvesterPlugin::canMine(Cell* cell) const
 {
- if (canMineMinerals() && cell->groundType() == Cell::GroundGrassMineral) {
+ if (canMineMinerals() && cell->hasMinerals() && cell->canGo(unitProperties())) {
 	return true;
  }
  // FIXME: there's no "oil cells" anymore
- if (canMineOil() && cell->groundType() == Cell::GroundGrassOil) {
+ if (canMineOil() && cell->hasOil() && cell->canGo(unitProperties())) {
 	return true;
  }
  return false;
@@ -621,3 +625,231 @@ unsigned int HarvesterPlugin::unloadingSpeed() const
  }
  return prop->unloadingSpeed();
 }
+
+
+BombingPlugin::BombingPlugin(Unit* owner) : UnitPlugin(owner)
+{
+ mWeapon = 0;
+ mPosX = 0;
+ mPosY = 0;
+}
+
+BombingPlugin::~BombingPlugin()
+{
+}
+
+void BombingPlugin::bomb(int weaponId, float x, float y)
+{
+ boDebug() << k_funcinfo << "wep: " << weaponId << "; pos: (" << x << "; " << y << ")" << endl;
+ BosonWeapon* w = unit()->weapon(weaponId);
+ if (!w) {
+	boError() << k_funcinfo << "No weapon with id " << weaponId << endl;
+	return;
+ }
+ if (w->properties()->shotType() != BosonShot::Bomb) {
+	boError() << k_funcinfo << "Weapon with id " << weaponId << " is not a bomb" << endl;
+	return;
+ }
+
+ unit()->stopMoving();
+
+ int cellX = (int)(x / BO_TILE_SIZE);
+ int cellY = (int)(y / BO_TILE_SIZE);
+ if (!canvas()->cell(cellX, cellY)) {
+	boError() << k_funcinfo << x << "," << y << " is no valid cell!" << endl;
+	return;
+ }
+
+ // This is where unit has to be when making the drop
+ // FIXME: REWRITE MOVING CODE OF Unit!!!
+ // This is probably one of the worst examples of hacks used to work around
+ //  problems with moving code. I want to be able to just write
+ //  unit()->move(x, y) and unit would go exactly to (x, y) without any trouble.
+ //  But ATM this isn't possible.
+ mPosX = (int)(cellX * BO_TILE_SIZE + BO_TILE_SIZE / 2 - unit()->width() / 2/* + w->offset().x()*/);
+ mPosY = (int)(cellY * BO_TILE_SIZE + BO_TILE_SIZE / 2 - unit()->height() / 2/* + w->offset().y()*/);
+ boDebug() << k_funcinfo << "Drop-point: (" << mPosX << "; " << mPosY << ")" << endl;
+ mWeapon = w;
+
+ unit()->setPluginWork(UnitPlugin::Bomb);
+}
+
+void BombingPlugin::advance(unsigned int)
+{
+ boDebug() << k_funcinfo << endl;
+
+ // Check if we're at the drop point
+ float dist = QMAX(QABS(unit()->x() - mPosX), QABS(unit()->y() - mPosY));
+ boDebug() << k_funcinfo << "dist: " << dist << endl;
+ boDebug() << k_funcinfo << "my pos is: (" << unit()->x() << "; " << unit()->y() << ");  drop-point is: (" << mPosX << "; " << mPosX << ")" << endl; // ";  movedest is: (" << d->mMoveDestX << "; " << d->mMoveDestY << ")" << endl;
+// int x = (int)(BosonItem::x() + width() / 2);
+// int y = (int)(BosonItem::y() + height() / 2);
+ if ((unit()->x() != mPosX) || (unit()->y() != mPosY)) {
+// if ((x() != d->mMoveDestX) || (y() != d->mMoveDestY)) {
+// if (dist > 2) {
+	boDebug() << k_funcinfo << "not at drop point - moving..." << endl;
+	if (!unit()->moveTo(mPosX, mPosY, 0, false, false)) {
+		boWarning() << k_funcinfo << "Moving failed. Now what?" << endl;
+		unit()->setWork(Unit::WorkNone);
+	} else {
+		//d->mSlowDownAtDestination = 0;
+		unit()->setAdvanceWork(Unit::WorkMove);
+	}
+	return;
+ }
+
+ // We're at drop point. Drop the bomb
+ boDebug() << k_funcinfo << "At drop-point!" << endl;
+ if (mWeapon->reloaded()) {
+	boDebug() << k_funcinfo << "Bomb ready. Dropping..." << endl;
+	mWeapon->dropBomb();
+	// And get the hell out of there
+	// Go away from bomb's explosion radius
+	float dist = mWeapon->properties()->damageRange() * BO_TILE_SIZE + unit()->width() / 2;
+  boDebug() << k_funcinfo << "Getaway dist: " << dist << "; rot: " << unit()->rotation() << endl;
+	float newx = unit()->x();
+	float newy = unit()->y();
+	// TODO: quite messy code, maybe it can be cleaned up somehow
+	int rot = (int)unit()->rotation() % 360;
+	if (rot >= 45 && rot <= 135) {
+		newx += dist;
+	} else if (rot >= 225 && rot <= 315) {
+		newx -= dist;
+	}
+	if (rot <= 45 || rot >= 315) {
+		newy -= dist;
+	} else if (rot >= 135 && rot <= 225) {
+		newy += dist;
+	}
+
+	// Make sure coords are valid
+	newx = QMAX(0, QMIN(newx, (canvas()->mapWidth() - 1) * BO_TILE_SIZE));
+	newy = QMAX(0, QMIN(newy, (canvas()->mapHeight() - 1) * BO_TILE_SIZE));
+
+  boDebug() << k_funcinfo << "Getaway point is at (" << newx << "; " << newy << ")" << endl;
+	if (!unit()->moveTo(newx, newy)) {
+		boWarning() << k_funcinfo << "Aargh! Can't move away from drop-point!" << endl;
+		unit()->setWork(Unit::WorkNone);
+	} else {
+		unit()->setWork(Unit::WorkMove);  // We don't want to return here anymore
+	}
+	mWeapon = 0;
+	boDebug() << k_funcinfo << "returning" << endl;
+	return;
+ }
+}
+
+
+MiningPlugin::MiningPlugin(Unit* owner) : UnitPlugin(owner)
+{
+ mWeapon = 0;
+ mPlacingCounter = 0;
+}
+
+MiningPlugin::~MiningPlugin()
+{
+}
+
+// How many game ticks does it take to place mine
+// FIXME: make configurable in BosonWeapon
+#define MINE_PLACE_TIME 10
+
+void MiningPlugin::mine(int weaponId)
+{
+ boDebug() << k_funcinfo << "wep: " << weaponId << endl;
+ BosonWeapon* w = unit()->weapon(weaponId);
+ if (!w) {
+	boError() << k_funcinfo << "No weapon with id " << weaponId << endl;
+	return;
+ }
+ if (w->properties()->shotType() != BosonShot::Mine) {
+	boError() << k_funcinfo << "Weapon with id " << weaponId << " is not a mine" << endl;
+	return;
+ }
+
+ unit()->stopMoving();
+
+ mWeapon = w;
+ mPlacingCounter = MINE_PLACE_TIME;
+
+ // Mine will be layed in advance()
+
+ unit()->setPluginWork(UnitPlugin::Mine);
+}
+
+void MiningPlugin::advance(unsigned int)
+{
+ boDebug() << k_funcinfo << endl;
+
+ // Lay the mine
+ if (mWeapon->reloaded()) {
+	// Don't place the mine immediately
+	if (mPlacingCounter > 0) {
+		boDebug() << k_funcinfo << "mPlacingCounter: " << mPlacingCounter << endl;
+		mPlacingCounter--;
+		return;
+	}
+
+	boDebug() << k_funcinfo << "Mine ready. Placing..." << endl;
+	mWeapon->layMine();
+
+	// Go one cell away from the mine. Maybe go away from explosion radius?
+	// FIXME: code taken from BombingPlugin. This could probably be written better
+	float dist = 1 * BO_TILE_SIZE + unit()->width() / 2;
+	boDebug() << k_funcinfo << "Getaway dist: " << dist << "; rot: " << unit()->rotation() << endl;
+	float oldx = unit()->x();
+	float oldy = unit()->y();
+	float newx, newy;
+	bool couldmove = false;
+
+	// TODO: quite messy code, maybe it can be cleaned up somehow
+	for (int i = 0; i <= 7; i++) {
+		newx = oldx;
+		newy = oldy;
+
+		// First try to go straight ahead, then try go to 45 degrees right, then
+		//  45 degrees left, then 2 * 45 degrees right, then 2 * 45 degrees left, etc
+		int rotadd = ((i + 1) / 2) * 45;
+		if (i % 2 == 0) {
+			rotadd = -rotadd;
+		}
+		int rot = ((int)unit()->rotation() + rotadd) % 360;
+
+		boDebug() << k_funcinfo << "i: " << i << "; rotadd: " << rotadd << "; rot: " << rot << endl;
+
+		if (rot >= 45 && rot <= 135) {
+			newx += dist;
+		} else if (rot >= 225 && rot <= 315) {
+			newx -= dist;
+		}
+		if (rot <= 45 || rot >= 315) {
+			newy -= dist;
+		} else if (rot >= 135 && rot <= 225) {
+			newy += dist;
+		}
+
+		// Make sure coords are valid
+		newx = QMAX(0, QMIN(newx, (canvas()->mapWidth() - 1) * BO_TILE_SIZE));
+		newy = QMAX(0, QMIN(newy, (canvas()->mapHeight() - 1) * BO_TILE_SIZE));
+
+		boDebug() << k_funcinfo << "i: " << i << "; Getaway point is at (" << newx << "; " << newy << ")" << endl;
+		if (unit()->moveTo(newx, newy)) {
+			unit()->setWork(Unit::WorkMove);  // We don't want to return here anymore
+			couldmove = true;
+			break;
+		}
+	}
+
+	if (!couldmove) {
+		boDebug() << k_funcinfo << "Can't move away!" << endl;
+		unit()->setWork(Unit::WorkNone);  // We don't want to return here anymore
+	}
+
+	mWeapon = 0;
+	boDebug() << k_funcinfo << "returning" << endl;
+	return;
+ } else {
+	boDebug() << k_funcinfo << "Weapon not yet reloaded" << endl;
+ }
+}
+
