@@ -19,15 +19,19 @@
 #ifndef UNITPROPERTIES_H
 #define UNITPROPERTIES_H
 
+#include "global.h"
+
 class SpeciesTheme;
 class PluginProperties;
 class BosonWeaponProperties;
 class BosonParticleSystem;
 class BosonParticleSystemProperties;
 class BoVector3;
+class BoAction;
 class QString;
 template<class T> class QValueList;
 template<class T> class QPtrList;
+template<class T> class QIntDict;
 template<class T1, class T2> class QMap;
 
 class KSimpleConfig;
@@ -120,10 +124,9 @@ public:
 	unsigned int unitHeight() const { return mUnitHeight; }
 
 	/**
-	 * @return The unrotated height in z-direction of the unit. The value
-	 * is number of cells this unit occupies * BO_TILE_SIZE
+	 * @return The unrotated height in z-direction of the unit.
 	 **/
-	unsigned int unitDepth() const { return mUnitDepth; }
+	float unitDepth() const { return mUnitDepth; }
 
 	/**
 	 * @return Default health aka power aka hitpoints of this unit.
@@ -182,11 +185,29 @@ public:
 	bool isFacility() const;
 
 	/**
-	 * @return The speed of the mobiel unit. 0 if this is a facility. See
+	 * @return Maximal speed of the mobile unit. 0 if this is a facility. See
 	 * @ref isFacility
 	 **/
 	float speed() const;
 
+	/**
+	 * @return How fast this mobile unit accelerates. 0 if this is a facility. See
+	 * @ref isFacility
+	 **/
+	float accelerationSpeed() const;
+
+	/**
+	 * @return How fast this mobile unit decelerates. 0 if this is a facility. See
+	 * @ref isFacility
+	 **/
+	float decelerationSpeed() const;
+
+	/**
+	 * @return Turning speed of this mobile unit (degrees per advance call). 0 if
+	 * this is a facility. See @ref isFacility
+	 **/
+	int rotationSpeed() const;
+
 	/**
 	 * @return Whether this unit can go over land
 	 **/
@@ -329,6 +350,29 @@ public:
 	 **/
 	const BosonWeaponProperties* weaponProperties(unsigned long int id) const;
 
+	BoAction* action(UnitAction type) const;
+
+	const QIntDict<BoAction>* allActions() const;
+
+	BoAction* produceAction() const { return mProduceAction; };
+
+	unsigned int explodingFragmentCount() const { return mExplodingFragmentCount; };
+	long int explodingFragmentDamage() const { return mExplodingFragmentDamage; };
+	float explodingFragmentDamageRange() const { return mExplodingFragmentDamageRange; };
+	QPtrList<BosonParticleSystem> newExplodingFragmentFlyParticleSystems(BoVector3 pos) const;
+	QPtrList<BosonParticleSystem> newExplodingFragmentHitParticleSystems(BoVector3 pos) const;
+
+	bool removeWreckageImmediately() const { return mRemoveWreckageImmediately; }
+
+
+	/**
+	 * Load actions for this unit. Must be called after overview pixmaps are
+	 * loaded.
+	 * Should be used by only SpeciesTheme.
+	 **/
+	void loadActions();
+
+
 protected:
 	void loadMobileProperties(KSimpleConfig* conf);
 	void loadFacilityProperties(KSimpleConfig* conf);
@@ -348,10 +392,10 @@ protected:
 	// Methods to set values. They are only meant to be used by upgrades and unit
 	//  editor. Don't use them unless you know what you are doing
 	void setName(const QString& name);
-	void setId(unsigned long int id)  { mTypeId = id; };
+	void setTypeId(unsigned long int id)  { mTypeId = id; };
 	void setUnitWidth(unsigned int unitWidth)  { mUnitWidth = unitWidth; };
 	void setUnitHeight(unsigned int unitHeight)  { mUnitHeight = unitHeight; };
-	void setUnitDepth(unsigned int unitDepth)  { mUnitDepth = unitDepth; };
+	void setUnitDepth(float unitDepth)  { mUnitDepth = unitDepth; };
 	void setHealth(unsigned long int health)  { mHealth = health; };
 	void setSightRange(unsigned int sightRange)  { mSightRange = sightRange; };
 	void setProducer(unsigned int producer)  { mProducer = producer; };
@@ -368,10 +412,14 @@ protected:
 	void setExplodingDamageRange(float range)  { mExplodingDamageRange = range; };
 	void setExplodingDamage(long int damage)  { mExplodingDamage = damage; };
 	void setHitPoint(const BoVector3& hitpoint);
+	void setRemoveWreckageImmediately(bool remove)  { mRemoveWreckageImmediately = remove; }
 
 	// These only have effect if there is mobile or facility properties
 	void setConstructionSteps(unsigned int steps);
 	void setSpeed(float speed);
+	void setAccelerationSpeed(float speed);
+	void setDecelerationSpeed(float speed);
+	void setRotationSpeed(int speed);
 	void setCanGoOnLand(bool c);
 	void setCanGoOnWater(bool c);
 
@@ -407,7 +455,7 @@ private:
 	unsigned long int mTypeId; // note: 0 is invalid!
 	unsigned int mUnitWidth;
 	unsigned int mUnitHeight;
-	unsigned int mUnitDepth;
+	float mUnitDepth;
 	unsigned long int mHealth;
 	unsigned int mSightRange;
 	unsigned int mProducer;
@@ -425,6 +473,11 @@ private:
 	unsigned long int mMaxAirWeaponRange;
 	long int mExplodingDamage;
 	float mExplodingDamageRange;
+	BoAction* mProduceAction;
+	unsigned int mExplodingFragmentCount;
+	long int mExplodingFragmentDamage;
+	float mExplodingFragmentDamageRange;
+	bool mRemoveWreckageImmediately;
 
 	MobileProperties* mMobileProperties;
 	FacilityProperties* mFacilityProperties;
